@@ -44,7 +44,7 @@ class HANDLER:
 
 
 class SlashCommandResponse:
-    def __init__(self, client, func, name):
+    def __init__(self, client, func, name: str, description: str=None, options: list=None, guild_ids: list=None):
         self.client = client
         if hasattr(func, '__slash_checks__'):
             self.checks = func.__slash_checks__
@@ -58,6 +58,13 @@ class SlashCommandResponse:
             self._buckets = CooldownMapping(cooldown)
         self.name = name
         self.func = func
+        self.guild_ids = guild_ids
+        if description is not None:
+            self.registerable = SlashCommand(name, description, options)
+        elif options is not None:
+            raise SyntaxError('<options> require <description> specified')
+        else:
+            self.registerable = None
     
     async def __call__(self, interaction):
         cog = get_class(self.func)
@@ -127,26 +134,40 @@ class NotOwner(SlashCommandError):
 #-----------------------------------+
 def command(*args, **kwargs):
     '''
-    A decorator that registers a function below as response for specified slash-command
+    A decorator that registers a function below as response for specified slash-command.
 
-    `name` - name of the slash-command you want to respond to
-    (equals to function name by default)
+    Parameters are similar to SlashCommand arguments.
 
-    (defaults to `None`, in this case function responds to
-    both global and local commands with the same names)
+    If ``description`` is specified, the decorator will be interpreted as SlashCommand and
+    will be registered (or edited) automatically with the given set of arguments.
 
-    ## Example 
-    ```
-    @slash_commands.command(name='user-info')
-    async def user_info(interaction):
-        # Your code
-    ```
+    Parameters
+    ----------
+
+    name : str
+        (optional) name of the slash-command you want to respond to (equals to function name by default)
+    
+    description : str
+        (optional) if specified, the client will automatically register a command with this description
+    
+    options : List[Option]
+        (optional) if specified, the client will
+        automatically register a command with this list of options. Requires ``description``
+    
+    guild_ids : List[int]
+        (optional) if specified, the client will register a command in these guilds.
+        Otherwise this command will be registered globally. Requires ``description``
     '''
     def decorator(func):
         if not asyncio.iscoroutinefunction(func):
             raise TypeError(f'<{func.__qualname__}> must be a coroutine function')
         name = kwargs.get('name', func.__name__)
-        new_func = SlashCommandResponse(HANDLER.client, func, name)
+        new_func = SlashCommandResponse(
+            HANDLER.client, func, name,
+            kwargs.get('description'),
+            kwargs.get('options'),
+            kwargs.get('guild_ids')
+        )
         HANDLER.commands[name] = new_func
         return new_func
     return decorator
@@ -308,6 +329,7 @@ class SlashClient:
         self.client = HANDLER.client
         self.events = {}
         self.registered_global_commands = []
+        self.registered_guild_commands = {}
         self.active_shard_count = 0
         self.is_ready = False
         self.client.add_listener(self._on_shard_connect, 'on_shard_connect')
@@ -339,24 +361,84 @@ class SlashClient:
 
     def command(self, *args, **kwargs):
         '''
-        A decorator that registers a function below as response for specified slash-command
+        A decorator that registers a function below as response for specified slash-command.
+
+        Parameters are similar to SlashCommand arguments.
+
+        If ``description`` is specified, the decorator will be interpreted as SlashCommand and
+        will be registered (or edited) automatically with the given set of arguments.
 
         Parameters
         ----------
 
         name : str
-            name of the slash-command you want to response to (equals to function name by default)
+            (optional) name of the slash-command you want to respond to (equals to function name by default)
+        
+        description : str
+            (optional) if specified, the client will automatically register a command with this description
+        
+        options : List[Option]
+            (optional) if specified, the client will
+            automatically register a command with this list of options. Requires ``description``
+        
+        guild_ids : List[int]
+            (optional) if specified, the client will register a command in these guilds.
+            Otherwise this command will be registered globally. Requires ``description``
         '''
         def decorator(func):
             if not asyncio.iscoroutinefunction(func):
                 raise TypeError(f'<{func.__qualname__}> must be a coroutine function')
             name = kwargs.get('name', func.__name__)
-            new_func = SlashCommandResponse(self.client, func, name)
+            new_func = SlashCommandResponse(
+                self.client, func, name,
+                kwargs.get('description'),
+                kwargs.get('options'),
+                kwargs.get('guild_ids')
+            )
             self.commands[name] = new_func
             return new_func
         return decorator
     
-    # Working with slash-commands
+    # Getters
+    def get_global_command(self, command_id: int):
+        """
+        Get a cached global command
+
+        Parameters
+        ----------
+
+        command_id : int
+            the ID of the command
+        
+        Returns
+        -------
+
+        slash_command : SlashCommand | None
+        """
+        for cmd in self.registered_global_commands:
+            if cmd.id == command_id:
+                return cmd
+    
+    def get_global_command_named(self, name: str):
+        """
+        Get a cached global command matching the specified name
+
+        Parameters
+        ----------
+
+        name : str
+            the name of the command
+        
+        Returns
+        -------
+
+        slash_command : SlashCommand | None
+        """
+        for cmd in self.registered_global_commands:
+            if cmd.name == name:
+                return cmd
+
+    # Straight references to API
     async def fetch_global_commands(self):
         '''Requests a list of global registered commands from the API
 
@@ -439,10 +521,13 @@ class SlashClient:
         '''
         if not isinstance(slash_command, SlashCommand):
             raise ValueError('Expected <SlashCommand> instance')
-        await self.client.http.request(
+        r = await self.client.http.request(
             Route('POST', '/applications/{app_id}/commands', app_id=self.client.user.id),
             json=slash_command.to_dict()
         )
+        sc = SlashCommand.from_dict(r)
+        self.registered_global_commands.append(sc)
+        return sc
     
     async def register_guild_slash_command(self, guild_id: int, slash_command: SlashCommand):
         '''Registers a local slash-command
@@ -484,13 +569,20 @@ class SlashClient:
             if cmd.id == command_id:
                 self.registered_global_commands[i] = slash_command
                 break
-        await self.client.http.request(
+        r = await self.client.http.request(
             Route(
                 'PATCH', '/applications/{app_id}/commands/{cmd_id}',
                 app_id=self.client.user.id, cmd_id=command_id
             ),
             json=slash_command.to_dict()
         )
+        # Update cache
+        sc = SlashCommand.from_dict(r)
+        for i, cmd in enumerate(self.registered_global_commands):
+            if cmd.id == command_id:
+                self.registered_global_commands[i] = sc
+                break
+        return sc
     
     async def edit_guild_slash_command(self, guild_id: int, command_id: int, slash_command: SlashCommand):
         '''Edits a local command
@@ -533,6 +625,11 @@ class SlashClient:
                 app_id=self.client.user.id, cmd_id=command_id
             )
         )
+        # Update cache
+        for i, cmd in enumerate(self.registered_global_commands):
+            if cmd.id == command_id:
+                self.registered_global_commands.pop(i)
+                break
     
     async def delete_guild_slash_command(self, guild_id: int, command_id: int):
         '''Deletes a local command
@@ -551,7 +648,7 @@ class SlashClient:
             )
         )
 
-    # Slower methods
+    # Even slower API methods
     async def fetch_global_command_named(self, name: str):
         '''
         Fetches a global command that matches the specified name
@@ -584,7 +681,12 @@ class SlashClient:
         name : str
             the name of the command to fetch
         '''
+        if guild_id in self.registered_guild_commands:
+            for cmd in self.registered_guild_commands[guild_id]:
+                if cmd.name == name:
+                    return cmd
         cmds = await self.fetch_guild_commands(guild_id)
+        self.registered_guild_commands[guild_id] = cmds
         for cmd in cmds:
             if cmd.name == name:
                 return cmd
@@ -657,12 +759,38 @@ class SlashClient:
         if cmd is not None:
             await self.delete_guild_command(guild_id, cmd.id)
 
-    # Internal use only
+    # Internal things
+    # Mega automated super-smart AI powered destructor-2000
+    async def _auto_register_or_patch(self):
+        for cmd in HANDLER.commands.values():
+            if cmd.registerable is not None:
+                # Local registration
+                if cmd.guild_ids is not None:
+                    # Iterate through guilds
+                    for ID in cmd.guild_ids:
+                        # Check if the command is registered
+                        old_cmd = await self.fetch_guild_command_named(ID, cmd.name)
+                        if old_cmd is None:
+                            await self.register_guild_slash_command(ID, cmd.registerable)
+                        elif not (old_cmd == cmd.registerable):
+                            delattr(cmd.registerable, 'name')
+                            await self.edit_guild_slash_command(ID, old_cmd.id, cmd.registerable)
+                # Global registration
+                else:
+                    old_cmd = await self.fetch_global_command_named(cmd.name)
+                    if old_cmd is None:
+                        await self.register_global_slash_command(cmd.registerable)
+                    elif not (old_cmd == cmd.registerable):
+                        delattr(cmd.registerable, 'name')
+                        await self.edit_global_slash_command(old_cmd.id, cmd.registerable)
+
+    # Adding relevant listeners
     async def _on_shard_connect(self, shard_id):
         self.client._AutoShardedClient__shards[shard_id].ws._discord_parsers['INTERACTION_CREATE'] = self._do_invokation
         self.active_shard_count += 1
         if self.active_shard_count >= self.client.shard_count:
             self.registered_global_commands = await self.fetch_global_commands()
+            await self._auto_register_or_patch()
             self.is_ready = True
             await self._activate_event('ready')
     
@@ -670,6 +798,7 @@ class SlashClient:
         if not isinstance(self.client, AutoShardedClient):
             self.client.ws._discord_parsers['INTERACTION_CREATE'] = self._do_invokation
             self.registered_global_commands = await self.fetch_global_commands()
+            await self._auto_register_or_patch()
             self.is_ready = True
             await self._activate_event('ready')
     
